@@ -119,7 +119,7 @@ void          pygobject_watch_closure    (PyObject *self, GClosure *closure);
 void          pygobject_register_sinkfunc(GType type,
 					  void (* sinkfunc)(GObject *object));
 int           pyg_type_register          (PyTypeObject *class,
-					  char *typename);
+					  const gchar *typename);
 
 /* from pygboxed.c */
 extern PyTypeObject PyGBoxed_Type;
