@@ -558,6 +558,8 @@ pygobject_new_with_interfaces(GType gtype)
     PyObject *modules, *module;
     gchar *type_name, *mod_name, *gtype_name;
 
+    state = pyg_gil_state_ensure();
+
     interfaces = g_type_interfaces (gtype, &n_interfaces);
     bases = PyTuple_New(n_interfaces+1);
     
@@ -566,6 +568,7 @@ pygobject_new_with_interfaces(GType gtype)
     py_parent_type = pygobject_lookup_class(parent_type);
 
     /* We will always put the parent at the first position in bases */
+    Py_INCREF(py_parent_type); /* PyTuple_SetItem steals a reference */
     PyTuple_SetItem(bases, 0, (PyObject*)py_parent_type);
 
     /* And traverse interfaces */
@@ -573,6 +576,7 @@ pygobject_new_with_interfaces(GType gtype)
 	for (i = 0; i < n_interfaces; i++) {
 	    interface_type = interfaces[i];
 	    py_interface_type = pygobject_lookup_class(interface_type);
+            Py_INCREF(py_interface_type); /* PyTuple_SetItem steals a reference */
 	    PyTuple_SetItem(bases, i+1, (PyObject*)py_interface_type);
 	}
 	
@@ -611,16 +615,14 @@ pygobject_new_with_interfaces(GType gtype)
 	type_name = g_strconcat(mod_name, ".", gtype_name, NULL);
     }
 
-    state = pyg_gil_state_ensure();
 
-    type = (PyTypeObject*)PyObject_CallFunction((PyObject*)&PyType_Type, "sOO",
+    type = (PyTypeObject*)PyObject_CallFunction((PyObject*)&PyType_Type, "sNN",
 						type_name, bases, dict);
     g_free(type_name);
 
-    pyg_gil_state_release(state);
-    
     if (type == NULL) {
 	PyErr_Print();
+        pyg_gil_state_release(state);
 	return NULL;
     }
 
@@ -637,21 +639,23 @@ pygobject_new_with_interfaces(GType gtype)
 
     if (PyType_Ready(type) < 0) {
 	g_warning ("couldn't make the type `%s' ready", type->tp_name);
+        pyg_gil_state_release(state);
 	return NULL;
     }
     
     /* insert type name in module dict */
     modules = PyImport_GetModuleDict();
     if ((module = PyDict_GetItemString(modules, mod_name)) != NULL) {
-	PyObject *mod_dict = PyModule_GetDict(module);
-	if (mod_dict != NULL)
-	    PyDict_SetItemString(mod_dict, gtype_name, (PyObject *)type);
+        if (PyObject_SetAttrString(module, gtype_name, (PyObject *)type) < 0)
+            PyErr_Clear();
     }
 
     /* stash a pointer to the python class with the GType */
     Py_INCREF(type);
     g_type_set_qdata(gtype, pygobject_class_key, type);
 
+    pyg_gil_state_release(state);
+
     return type;
 }
 
@@ -1608,30 +1612,28 @@ PyTypeObject PyGObject_Type = {
 
 
 static int
-pygobjectmeta_register(PyTypeObject *subtype, PyObject *instance_dict)
+pygobjectmeta_register(PyTypeObject *subtype)
 {
     PyObject *pytype_name;
-    char *type_name = NULL;
+    const char *type_name = NULL;
     int retval = 0;
     
-    if (!instance_dict) {
-        PyErr_Clear();
-	goto out;
-    }
-
     /* Maybe the type doesn't really need to registered? */
-    if (!(PyDict_GetItemString(instance_dict, "__gtype_name__")
-          || PyDict_GetItemString(instance_dict, "__gproperties__")
-          || PyDict_GetItemString(instance_dict, "__gsignals__")))
+    if (subtype->tp_dict == NULL)
+	goto out;
+	
+    if (!(PyDict_GetItemString(subtype->tp_dict, "__gtype_name__")
+          || PyDict_GetItemString(subtype->tp_dict, "__gproperties__")
+          || PyDict_GetItemString(subtype->tp_dict, "__gsignals__")))
 	goto out;
 
     /* If it's already registered, skip registration */
-    if (PyDict_GetItemString(instance_dict, "__gtype__"))
+    if (PyDict_GetItemString(subtype->tp_dict, "__gtype__"))
 	goto out;
 
-    pytype_name = PyDict_GetItemString(instance_dict, "__gtype_name__");
+    pytype_name = PyDict_GetItemString(subtype->tp_dict, "__gtype_name__");
     if (pytype_name)
-	type_name = g_strdup(PyString_AsString(pytype_name));
+	type_name = PyString_AsString(pytype_name);
     retval = pyg_type_register(subtype, type_name);
 
 out:
@@ -1642,13 +1644,10 @@ out:
 static int
 pygobjectmeta_init(PyTypeObject *subtype, PyObject *args, PyObject *kwargs)
 {
-    PyObject *instance_dict;
-    
     if (PyType_Type.tp_init((PyObject *) subtype, args, kwargs))
         return -1;
     
-    instance_dict = PyTuple_GetItem(args, 2);
-    return pygobjectmeta_register(subtype, instance_dict);
+    return pygobjectmeta_register(subtype);
 }
 
 
@@ -1686,7 +1685,7 @@ PyTypeObject PyGObject_MetaType = {
     0,                                  /* tp_methods */
     0,					/* tp_members */
     0,					/* tp_getset */
-    &PyType_Type,			/* tp_base */
+    NULL,				/* tp_base */
     (PyObject *)0,			/* tp_dict */
     0,					/* tp_descr_get */
     0,					/* tp_descr_set */
